"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useBalances = void 0;
const contexts_1 = require("../contexts");
const objects_1 = require("../interfaces/objects");
const react_1 = require("react");
const react_alert_1 = require("react-alert");
const constants_1 = require("../constants");
var EActionTypes;
(function (EActionTypes) {
    EActionTypes["FETCHING"] = "FETCHING";
    EActionTypes["FETCHED"] = "FETCHED";
    EActionTypes["ERROR"] = "ERROR";
    EActionTypes["TOKEN_SELECTED"] = "TOKEN_SELECTED";
})(EActionTypes || (EActionTypes = {}));
// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
function useBalances() {
    const { coreProxy } = contexts_1.useStoreContext();
    const { setLoading } = contexts_1.useLayoutContext();
    const alert = react_alert_1.useAlert();
    react_1.useEffect(() => {
        let cancelRequest = false;
        const fetchData = async () => {
            if (cancelRequest)
                return;
            dispatch({ type: EActionTypes.FETCHING });
            setLoading(true);
            coreProxy
                ?.getBalances()
                .map((balance) => {
                setLoading(false);
                dispatch({
                    type: EActionTypes.FETCHED,
                    payload: balance,
                });
            })
                .mapErr((error) => {
                setLoading(false);
                alert.error(error.message || "An error has happened while pulling balances");
                dispatch({ type: EActionTypes.ERROR, payload: error.message });
            });
        };
        fetchData();
        coreProxy?.onBalancesChanged.subscribe({
            next: (balance) => {
                if (cancelRequest)
                    return;
                dispatch({
                    type: EActionTypes.FETCHED,
                    payload: balance,
                });
            },
        });
        return function cleanup() {
            cancelRequest = true;
        };
    }, []);
    const initialState = {
        error: false,
        loading: true,
        balances: [],
        channelTokenSelectorOptions: [],
        preferredPaymentToken: undefined,
    };
    const [state, dispatch] = react_1.useReducer((state, action) => {
        switch (action.type) {
            case EActionTypes.FETCHING:
                return {
                    ...state,
                    error: false,
                    loading: true,
                };
            case EActionTypes.FETCHED:
                return {
                    ...state,
                    error: false,
                    loading: false,
                    balances: prepareBalances(action.payload),
                    channelTokenSelectorOptions: prepareChannelTokenSelectorOptions(action.payload),
                };
            case EActionTypes.TOKEN_SELECTED:
                return {
                    ...state,
                    error: false,
                    loading: false,
                    preferredPaymentToken: action.payload,
                };
            case EActionTypes.ERROR:
                return {
                    ...state,
                    error: true,
                    loading: false,
                };
            default:
                return state;
        }
    }, initialState);
    function prepareBalances(balance) {
        return balance.assets.reduce((acc, assetBalance) => {
            acc.push(assetBalance);
            return acc;
        }, []);
    }
    function prepareChannelTokenSelectorOptions(balance) {
        return balance.assets.reduce((acc, assetBalance) => {
            const tokenName = assetBalance.assetAddress === constants_1.ETHER_HEX_ADDRESS
                ? "ETH"
                : "HyperToken";
            acc.push(new objects_1.PaymentTokenOptionViewModel(tokenName, assetBalance.assetAddress));
            return acc;
        }, []);
    }
    return {
        ...state,
    };
}
exports.useBalances = useBalances;
//# sourceMappingURL=useBalances.js.map